// Return the nearest integer to the input.
template<class T>
int nint(T input)
{
    int result;
    if (input >= 0)
        result = int(input + 0.5);
    else
        result = int(input - 0.5);

    return result;
}

// Limit the input argument to the range [lower, upper].
template<class T>
T limit(T input, T lower, T upper)
{
    T result;
    if (input > upper)
        result = upper;
    else if (input < lower)
        result = lower;
    else
        result = input;

    return result;
}

// Quantize the input argument using a symmetric staircase function with step
// size equal to interval.
template<class T>
T quantize(T input, T interval)
{
    return interval * nint(input / interval);
}

// Delay the input by nframes. The input on the first frame will be the output
// for subsequent frames until the delay buffer has been filled one time.
template<class T, int nframes>
T transport_delay(T input)
{
    static T delay_buffer[nframes];
    static int buf_ptr = 0;

    if (buf_ptr == 0)
        for (int i=0; i<nframes; i++)
            delay_buffer[i] = input;

    int index = (buf_ptr++) % nframes;
    T result = delay_buffer[index];
    delay_buffer[index] = input;

    return result;
}