#include <Random.h>

#include <cmath>

double Random::Uniform(bool initialize, int seed)
{
    const int Mbig   = 1000000000;
    const int Mseed  =    1618033;

    int mj;

    if (initialize || !m_initialized)
    {
        m_fac = 1.0 / Mbig;
        mj = (Mseed - seed) % (Mbig);

        m_ma[Len-1] = mj;
        int mk = 1;
        for (int i=1; i<Len-1; i++)
        {
            int ii = (21*i) % (Len-1);
            m_ma[ii] = mk;
            mk = mj - mk;
            if (mk < 0)
                mk += Mbig;

            mj = m_ma[ii];
        }

        for (int k=1; k<5; k++)
            for (i=1; i<Len; i++)
            {
                m_ma[i] -= m_ma[1+(i+30) % (Len-1)];
                if (m_ma[i] < 0)
                    m_ma[i] += Mbig;
            }

        m_inext = 0;
        m_inextp = 31;
        m_initialized = true;
    }

    if (++m_inext == Len)
        m_inext = 1;

    if (++m_inextp == Len)
        m_inextp = 1;

    mj = m_ma[m_inext] - m_ma[m_inextp];
    if (mj < 0)
        mj += Mbig;

    m_ma[m_inext] = mj;
    return mj * m_fac;
}

double Random::Normal(bool initialize, int seed)
{
    if (!m_iset) // Generate a new pair
    {
        double v1, v2, r;
        do
        {
            v1 = 2.0*Uniform(initialize, seed) - 1.0;
            initialize = false;
            v2 = 2.0*Uniform(initialize, seed) - 1.0;

            r = v1*v1 + v2*v2;
        } while (r >= 1.0);

        double fac = sqrt(-2.0*log(r) / r);
        m_gset = v1*fac;
        m_iset = true;

        return v2*fac;
    }
    else
    {
        m_iset = false;
        return m_gset;
    }
}
