// Two dimensional interpolation

#include <dssl.h>

#include <cstdio>
#include <cassert>

int main()
{
    // Define the equally spaced breakpoints. The first argument is the 
    // first breakpoint value; the second is the breakpoint separation.
    EqSpacedBkpt<4> x_bkpt(3, 1), y_bkpt(1.2, 0.2);

    // Define a two-dimensional interpolation function with the above bkpts
    LinearInterp<2> interp;
    interp.SetDimension(0, &x_bkpt);
    interp.SetDimension(1, &y_bkpt);

    const double data[] =
    {
        0.0, 0.1, 0.3, 0.3,
        0.1, 0.3, 0.5, 0.4,
        0.1, 0.5, 0.6, 0.7,
        0.2, 0.5, 0.6, 0.9
    };

    interp.SetupData(data);

    // Open a file for the output data
    FILE* iov = fopen("z.txt", "w");
    assert(iov);

    // Interpolate the function across the ranges of the X and Y breakpoints
    // and write the results to the output file
    for (int ix=0; ix<=60; ix++)
    {
        double x = 3 + 0.05 * ix; // Function X input
        x_bkpt(x);
        for (int iy=0; iy<=60; iy++)
        {
            double y = 1.2 + 0.01 * iy; // Function Y input
            y_bkpt(y);

            double z = interp(); // Function interpolation result
            fprintf(iov, " %lf", z);
        }

        fprintf(iov, "\n");
    }

    fclose(iov);
    return 0;
}
