// Pendulum simulation
#include <dssl.h>

#include <cstdio>
#include <cmath>

int main()
{
    // Define the state variables
    StateList state_list;
    State<> theta(&state_list), theta_dot(&state_list);

    // Integration step size and simulation end time
    const double step_time = 0.01, end_time = 10.0;

    // Set the initial conditions
    theta.ic = 0.5;
    theta_dot.ic = 0.0;

    state_list.Initialize(step_time);

    // Pendulum model parameters
    const double g = 9.81;
    const double L = 1.0;

    // Open an output file
    FILE* iov = fopen("pendulum.csv", "w");
    assert(iov);

    fprintf(iov, "Time, Theta\n");
    for(;;)
    {
        // Pendulum dynamic equations
        theta_dot.der = -(g/L) * sin(theta);
        theta.der = theta_dot;

        fprintf(iov, "%6.2lf, %9.6lf\n", state_list.Time(), double(theta));

        if (state_list.Time() >= end_time)
            break;

        state_list.Integrate();
    }

    fclose(iov);
    return 0;
}
