#include "RigidBody.h"

StateList state_list;
RigidBody body(&state_list);

int main()
{
    // All states will be initialized to zero
    Vector<3> pos_ic, vel_ic, euler_ic, body_rate_ic;

    body.Initialize(pos_ic, vel_ic, euler_ic, body_rate_ic);

    const double step_time = 0.001, end_time = 1.0;

    state_list.Initialize(step_time);

    printf("Time, Px, Py, Pz, Vx, Vy, Vz, Phi, Theta, Psi, P, Q, R\n");
    for(;;)
    {
        // Set all accelerations to zero for now
        Vector<3> translational_accel, angular_accel;

        if (state_list.Time() <= 0.01) // If still in the barrel, accelerate
        {
            translational_accel[0] = 10000.0;
            angular_accel[0] = 1000.0;
        }

        body.Compute(translational_accel, angular_accel);

        // Print state information
        printf("%lf", state_list.Time());
        for (int i=0; i<3; i++) printf(",%lf", body.GetPos()[i]);
        for (i=0; i<3; i++) printf(",%lf", body.GetVel()[i]);
        for (i=0; i<3; i++) printf(",%lf", body.GetEuler()[i]);
        for (i=0; i<3; i++) printf(",%lf", body.GetBodyRate()[i]);
        printf("\n");

        if (state_list.Time() >= end_time)
            break;

        state_list.Integrate();
    }

    return 0;
}
