clear all
close all

tic;

J0 = 0.012;
b0 = 0.2;
K0 = 0.013;
R0 = 1.1;
L0 = 0.6;

J_sig = J0 / 3;
b_sig = b0 / 8;
K_sig = K0 / 8;
R_sig = R0 / 8;
L_sig = L0 / 5;

J = J0;
b = b0;
K = K0;
R = R0;
L = L0;

sim('Turntable')
Pos_nom = pos;
Dac_nom = dac;

i = 1
for iJ = -1:1
	J = J0 + 3*iJ*J_sig;
	for ib = -1:1
		b = b0 + 3*ib*b_sig;
		for iK = -1:1
			K = K0 + 3*iK*K_sig;
			for iR = -1:1
				R = R0 + 3*iR*R_sig;
				for iL = -1:1
					L = L0 + 3*iL*L_sig;
					sim('Turntable')
					pos_TIC(i) = theil(Pos_nom.signals.values, pos.signals.values);
					dac_TIC(i) = theil(Dac_nom.signals.values, dac.signals.values);
					param(i,1:7) = [i pos_TIC(i) iJ ib iK iR iL];
					i = i + 1
				end
			end
		end
	end
end

figure(1)
plot(pos_TIC, 'ko-')
xlabel('Run Number')
ylabel('Position TIC')
grid
fixup_plot(1)

figure(2)
plot(dac_TIC, 'ko-')
xlabel('Run Number')
ylabel('DAC Output Voltage TIC')
grid
fixup_plot(2)

param = sortrows(param, 2); % Sort from smallest TIC to largest
diff_rows = 10; % Number of rows to include in max_diff and min_diff

max_diff = param(i-1:-1:i-diff_rows, :)
mid_diff = param(i/2:i/2+diff_rows, :)
min_diff = param(1:diff_rows, :)

figure(3)
subplot(211)
plot(Pos_nom.time, Pos_nom.signals.values, 'k--')
hold on
for i=1:diff_rows
	J = J0 + max_diff(i, 3)*J_sig;
	b = b0 + max_diff(i, 4)*b_sig;
	K = K0 + max_diff(i, 5)*K_sig;
	R = R0 + max_diff(i, 6)*R_sig;
	L = L0 + max_diff(i, 7)*L_sig;
	sim('Turntable')
	subplot(211)
	plot(pos.time, pos.signals.values, 'k')
	hold on
	subplot(212)
	plot(pos.time, pos.signals.values-Pos_nom.signals.values, 'k')
	hold on
end
subplot(211)
grid on
axis([0 10 0 1.1])
xlabel('Time, seconds')
ylabel('Position, radians')
fixup_plot(3)
subplot(212)
grid on
axis([0 10 -0.25 0.25])
xlabel('Time, seconds')
ylabel('Position Difference, radians')
fixup_plot(3)

figure(4)
subplot(211)
plot(Pos_nom.time, Pos_nom.signals.values, 'k--')
hold on
for i=1:diff_rows
	J = J0 + mid_diff(i, 3)*J_sig;
	b = b0 + mid_diff(i, 4)*b_sig;
	K = K0 + mid_diff(i, 5)*K_sig;
	R = R0 + mid_diff(i, 6)*R_sig;
	L = L0 + mid_diff(i, 7)*L_sig;
	sim('Turntable')
	subplot(211)
	plot(pos.time, pos.signals.values, 'k')
	hold on
	subplot(212)
	plot(pos.time, pos.signals.values-Pos_nom.signals.values, 'k')
	hold on
end
subplot(211)
grid on
axis([0 10 0 1.1])
xlabel('Time, seconds')
ylabel('Position, radians')
fixup_plot(4)
subplot(212)
grid on
axis([0 10 -0.25 0.25])
xlabel('Time, seconds')
ylabel('Position Difference, radians')
fixup_plot(4)

figure(5)
subplot(211)
plot(Pos_nom.time, Pos_nom.signals.values, 'k--')
hold on
for i=1:diff_rows
	J = J0 + min_diff(i, 3)*J_sig;
	b = b0 + min_diff(i, 4)*b_sig;
	K = K0 + min_diff(i, 5)*K_sig;
	R = R0 + min_diff(i, 6)*R_sig;
	L = L0 + min_diff(i, 7)*L_sig;
	sim('Turntable')
	subplot(211)
	plot(pos.time, pos.signals.values, 'k')
	hold on
	subplot(212)
	plot(pos.time, pos.signals.values-Pos_nom.signals.values, 'k')
	hold on
end
subplot(211)
grid on
axis([0 10 0 1.1])
xlabel('Time, seconds')
ylabel('Position, radians')
fixup_plot(5)
subplot(212)
grid on
axis([0 10 -0.25 0.25])
xlabel('Time, seconds')
ylabel('Position Difference, radians')
fixup_plot(5)

corrcoef(pos_TIC, dac_TIC)

toc