function coef = theil(x,y)
%THEIL   Theil Inequality Coefficient
%   THEIL(X,Y) computes the Theil Inequality Coefficient
%   for two equal length vectors X and Y.
%   If X and Y are matrices, the comparison is done
%   column to column and the results are returned in
%   a row vector.

if nargin ~= 2
   error('Two input arguments are required to THEIL')
end

[rx, cx] = size(x);
[ry, cy] = size(y);
if (rx~=ry) | (cx~=cy) 
   error('The sizes of the X and Y inputs to THEIL must match')
end

if rx == 1 % Change row vectors to column vectors
   x = x(:);
   y = y(:);
	[rx, cx] = size(x);
	[ry, cy] = size(y);
end

for i=1:cx % for each column
   xi = x(:,i);
   yi = y(:,i);
   
   mx2 = mean(xi.^2);
   my2 = mean(yi.^2);
   mxy2 = mean((xi-yi).^2);
   
   num = sqrt(mxy2);
   den = sqrt(mx2) + sqrt(my2);
   
   if den ~= 0
	   coef(i) = num / den;
   else
	   coef(i) = 0; % Both data series are all zeros and agree exactly
   end
end
